package com.elinkthings.ailinksecrettooldemo;

import android.Manifest;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanFilter;
import android.bluetooth.le.ScanResult;
import android.bluetooth.le.ScanSettings;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.ParcelUuid;
import android.provider.Settings;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;

import com.elinkthings.ailinksecretlib.BleConfig;
import com.elinkthings.ailinksecretlib.MyBleDeviceUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.annotation.RequiresPermission;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;

public class MainActivity extends AppCompatActivity implements View.OnClickListener, OnAnalyticalListener {


    /**
     * 停止搜索
     */
    private final static int STOP_SEARCH = 2;
    private final static int REFRESH_DATA = 3;
    private final static int REFRESH_BLE_DATA = 4;


    private Button btn_search, btn_stop, btn_clear;
    private ListView listview1, listview2;

    /**
     * 蓝牙控制对象
     */
    private BluetoothManager mBleManager;
    private BluetoothAdapter mBluetoothAdapter;
    private Context mContext;
    private List<String> mList1 = new ArrayList<>();
    private ArrayAdapter listAdapter1;
    private List<String> mList2 = new ArrayList<>();
    private ArrayAdapter listAdapter2;

    private Handler mHandler = new Handler(Looper.getMainLooper()) {

        @Override
        public void handleMessage(@NonNull Message msg) {

            switch (msg.what) {

                case STOP_SEARCH:
                    stopScan();
                    break;

                case REFRESH_DATA:
                    if (listAdapter2 != null) {
                        listAdapter2.notifyDataSetChanged();
                    }
                    break;

                case REFRESH_BLE_DATA:
                    if (listAdapter1 != null) {
                        listAdapter1.notifyDataSetChanged();
                    }
                    break;

            }

        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mContext = this;
        init();

    }

    private void init() {
        initView();
        initData();
        initBle();
        initListener();


    }

    private void initData() {
        listAdapter1 = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, mList1);
        listview1.setAdapter(listAdapter1);

        listAdapter2 = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, mList2);
        listview2.setAdapter(listAdapter2);

    }

    @Override
    public void onClick(View v) {

        switch (v.getId()) {

            case R.id.btn_search:
                checkPermission();
                break;
            case R.id.btn_stop:
                stopScan();
                break;
            case R.id.btn_clear:
                mList2.clear();
                mHandler.sendEmptyMessage(REFRESH_DATA);
                break;


        }


    }

    /**
     * (16进制)
     * BLE蓝牙返回的byte[]
     * byte[]转字符串
     */
    public String byte2HexStr(byte[] b) {
        String hs = "";
        if (b == null) {
            return hs;
        }
        String stmp;
        for (byte aB : b) {
            int a = aB & 0XFF;
            stmp = Integer.toHexString(a);
            if (stmp.length() == 1)
                hs = hs + "0" + stmp + " ";
            else
                hs = hs + stmp + " ";
        }
        return hs;
    }

    @Override
    public void onAnalyticalData(String mac, UUID uuid, byte[] data) {
        mList2.add("收到的数据:\nmac=" + mac + "\ndata=" + byte2HexStr(data));
        mHandler.sendEmptyMessage(REFRESH_DATA);
    }

    @Override
    public void onHandshake(String mac, boolean result) {

        if (result) {
            mList2.add("握手成功:mac=" + mac);
        } else {
            mList2.add("握手失败:mac=" + mac);
        }
        mHandler.sendEmptyMessage(REFRESH_DATA);
    }

    private void initListener() {
        btn_search.setOnClickListener(this);
        btn_stop.setOnClickListener(this);
        btn_clear.setOnClickListener(this);
        listview1.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                stopScan();
                if (mList1.size() > position) {
                    if (mAnalyticalDataUtil != null) {
                        mAnalyticalDataUtil.close();
                    }
                    String mac = mList1.get(position);
                    connectBleDevice(mac);
                    mList2.add("正在连接:mac=" + mac);
                    mHandler.sendEmptyMessage(REFRESH_DATA);
                }

            }
        });

    }

    private void initView() {
        btn_search = findViewById(R.id.btn_search);
        btn_stop = findViewById(R.id.btn_stop);
        btn_clear = findViewById(R.id.btn_clear);
        listview1 = findViewById(R.id.listview1);
        listview2 = findViewById(R.id.listview2);

    }

    private void initBle() {
        if (mBleManager == null)
            mBleManager = (BluetoothManager) this.getSystemService(Context.BLUETOOTH_SERVICE);
        if (mBluetoothAdapter == null && mBleManager != null) {
            mBluetoothAdapter = mBleManager.getAdapter();
        }

    }


    /**
     * 开始搜索
     */
    private void startScan() {
        mList1.clear();
        mHandler.sendEmptyMessage(REFRESH_BLE_DATA);
        scanLeDevice(BleConfig.UUID_SERVER_AILINK);
        mHandler.sendEmptyMessageDelayed(STOP_SEARCH, 10 * 1000);
    }

    /**
     * 搜索设备
     * 扫描过于频繁会导致扫描失败
     * 需要保证5次扫描总时长超过30s
     *
     * @param scanUUID 过滤的UUID(空/null代码不过滤)
     */
    @RequiresPermission(allOf = {Manifest.permission.BLUETOOTH_ADMIN, Manifest.permission.BLUETOOTH})
    public void scanLeDevice(UUID... scanUUID) {
        if (!mBluetoothAdapter.isEnabled()) {
            //蓝牙未开启
            return;
        }


        try {
            //扫描过于频繁会导致扫描失败
            //需要保证5次扫描总时长超过30s
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                List<ScanFilter> filters = new ArrayList<>();
                if (scanUUID != null && scanUUID.length > 0) {
                    for (UUID uuid : scanUUID) {
                        ScanFilter filter = new ScanFilter.Builder().setServiceUuid(new ParcelUuid(uuid)).build();
                        filters.add(filter);
                    }
                }
                if (mScanCallback == null)
                    mScanCallback = new MyScanCallback();
                ScanSettings settings = new ScanSettings.Builder().setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY).build();
                mBluetoothAdapter.getBluetoothLeScanner().startScan(filters, settings, mScanCallback);
            } else {
                mBluetoothAdapter.startLeScan(mLeScanCallback);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    /**
     * 取消扫描
     */
    public void stopScan() {
        try {
            if (mBluetoothAdapter != null) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    if (mBluetoothAdapter.getBluetoothLeScanner() != null) {
                        mBluetoothAdapter.getBluetoothLeScanner().stopScan(mScanCallback);
                    }

                } else {
                    mBluetoothAdapter.stopLeScan(mLeScanCallback);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 搜索结果的回调
     */
    private BluetoothAdapter.LeScanCallback mLeScanCallback = new BluetoothAdapter.LeScanCallback() {
        @Override
        public void onLeScan(BluetoothDevice device, int rssi, byte[] scanRecord) {
            if (device == null || scanRecord == null)
                return;
            String address = device.getAddress();
            saveScanData(address);
        }
    };

    private MyScanCallback mScanCallback;


    /**
     * 搜索结果的回调 5.0以上
     */
    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    private class MyScanCallback extends ScanCallback {

        @Override
        public void onScanResult(int callbackType, ScanResult result) {
            super.onScanResult(callbackType, result);
            String address = result.getDevice().getAddress();
            saveScanData(address);
//            SparseArray<byte[]> manufacturerSpecificData = result.getScanRecord().getManufacturerSpecificData();//获取自定义厂商数据列表,正常情况只有1个
//            if (manufacturerSpecificData.size() > 0) {
//                byte[] data = manufacturerSpecificData.get(0);
//                decryptBroadcastExample(data);
//            }

        }

        @Override
        public void onBatchScanResults(List<ScanResult> results) {
            super.onBatchScanResults(results);

        }

        @Override
        public void onScanFailed(int errorCode) {
            super.onScanFailed(errorCode);
            //扫描失败

        }
    }


    /**
     * 解密广播数据参考例子
     * @param data 自定义厂商数据
     */
    private void decryptBroadcastExample(byte[] data){
           byte[] decryptData= mAnalyticalDataUtil.decryptBroadcast(data);//获取解密后的数据

    }


    /**
     * 搜索到的数据到列表
     */
    private void saveScanData(String mac) {
        if (!mList1.contains(mac)) {
            mList1.add(mac);
            mHandler.sendEmptyMessage(REFRESH_BLE_DATA);
        }
    }


    /**
     * 连接蓝牙设备
     *
     * @param mac 连接的设备地址
     */
    private synchronized void connectBleDevice(String mac) {
        if (!mBluetoothAdapter.isEnabled()) {
            //蓝牙未开启
            return;
        }


        BluetoothDevice device;
        try {
            device = mBluetoothAdapter.getRemoteDevice(mac);
            if (device == null) {
                //找不到需要连接的设备
                return;
            }
        } catch (IllegalArgumentException e) {
            //连接的设备地址无效
            e.printStackTrace();
            return;
        }

        device.connectGatt(mContext, false, mGattCallback);//连接操作

    }


    //------------------------------------------------------------------------------------------


    /**
     * 连接ble的回调操作类
     */
    private MyBluetoothGattCallback mGattCallback = new MyBluetoothGattCallback();
    ;
    private BluetoothGatt gattOld;
    private AnalyticalDataUtil mAnalyticalDataUtil;

    /**
     * 连接ble的操作回调类
     */
    private class MyBluetoothGattCallback extends BluetoothGattCallback {
        @Override
        public void onConnectionStateChange(final BluetoothGatt gatt, int status, int newState) {
            try {
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    if (newState == BluetoothProfile.STATE_CONNECTED && gattOld != gatt) {
                        gatt.discoverServices();
                        mList2.add("连接成功:mac=" + gatt.getDevice().getAddress());
                        mHandler.sendEmptyMessage(REFRESH_DATA);
                    } else if (newState == BluetoothProfile.STATE_DISCONNECTED) {
                        //避免系统多次回调断开连接的消息
                        mList2.add("连接断开:mac=" + gatt.getDevice().getAddress());
                        mHandler.sendEmptyMessage(REFRESH_DATA);
                        gatt.connect();
                    }
                } else {
                    mList2.add("连接断开:mac=" + gatt.getDevice().getAddress() + " || " + status);
                    mHandler.sendEmptyMessage(REFRESH_DATA);
                    gatt.disconnect();
                    MyBleDeviceUtils.refreshDeviceCache(gatt);
                }
            } catch (NullPointerException e) {
                e.printStackTrace();
            }

        }

        @Override
        public void onServicesDiscovered(final BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                gattOld = null;
                //发现新服务
                List<BluetoothGattService> mServices = gatt.getServices();

                if (mServices.size() > 0) {
                    mList2.add("获取服务成功:mac=" + gatt.getDevice().getAddress());
                    mHandler.sendEmptyMessage(REFRESH_DATA);
                    //获取的服务列表不为空
                    mAnalyticalDataUtil = new AnalyticalDataUtil(gatt, MainActivity.this);
                } else {
                    //连接失败:服务读取失败
                    mList2.add("获取服务失败:mac=" + gatt.getDevice().getAddress());
                    mHandler.sendEmptyMessage(REFRESH_DATA);
                    gatt.disconnect();
                    gatt.close();
                    MyBleDeviceUtils.refreshDeviceCache(gatt);
                }

            } else {
                //服务读取失败
                mList2.add("获取服务失败:mac=" + gatt.getDevice().getAddress());
                mHandler.sendEmptyMessage(REFRESH_DATA);
            }
        }


        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {

        }


        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {

        }

        @Override
        public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorRead(gatt, descriptor, status);
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorWrite(gatt, descriptor, status);
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            //通知返回的数据
            if (mAnalyticalDataUtil != null) {
                mAnalyticalDataUtil.notifyData(gatt, characteristic);
            }
        }


        @Override
        public void onReliableWriteCompleted(BluetoothGatt gatt, int status) {
            super.onReliableWriteCompleted(gatt, status);
        }

        @Override
        public void onReadRemoteRssi(BluetoothGatt gatt, int rssi, int status) {
            //回调信号强度

        }


        @Override
        public void onMtuChanged(BluetoothGatt gatt, int mtu, int status) {
            super.onMtuChanged(gatt, mtu, status);
            //TODO Mtu改变(一般用不上,可改变传送效率,兼容性待验)

        }


    }


    // ------------------- 权限 ------------------

    /**
     * 检查权限
     */
    private void checkPermission() {
        // 没有蓝牙权限就请求蓝牙权限
        if (!hasBluetooth()) {
            requestBluetooth();
            return;
        }
        // 没有定位权限就请求定位权限
        if (!hasLocationPermission()) {
            requestLocationPermission(this);
            return;
        }
        // 没有定位服务就请求定位服务
        if (!hasLocationService()) {
            requestLocationService();
            return;
        }
        // 都有了，OK
        startScan();
    }

    /**
     * 是否有定位权限
     *
     * @return boolean
     */
    private boolean hasLocationPermission() {
        return ActivityCompat.checkSelfPermission(mContext, Manifest.permission.ACCESS_FINE_LOCATION) == PackageManager.PERMISSION_GRANTED;
    }

    /**
     * 蓝牙是否打开
     *
     * @return boolean
     */
    private boolean hasBluetooth() {
        return BluetoothAdapter.getDefaultAdapter().isEnabled();
    }

    /**
     * 定位服务是否打开
     *
     * @return boolean
     */
    private boolean hasLocationService() {
        LocationManager locationManager = (LocationManager) mContext.getSystemService(Context.LOCATION_SERVICE);
        if (locationManager == null) {
            return false;
        }
        return locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
    }

    /**
     * 申请定位权限
     */
    private void requestLocationPermission(Activity activity) {
        ActivityCompat.requestPermissions(activity, new String[]{Manifest.permission.ACCESS_FINE_LOCATION}, 1);
    }

    /**
     * 申请打开蓝牙
     */
    private void requestBluetooth() {
        Intent intent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        startActivityForResult(intent, 1);
    }

    /**
     * 申请打开定位服务
     */
    private void requestLocationService() {
        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        startActivityForResult(intent, 2);
    }


}